@@ -2,7 +2,7 @@
  *
  * job_class.c - job class definition handling
  *
- * Copyright © 2010,2011 Canonical Ltd.
+ * Copyright © 2011 Canonical Ltd.
  * Author: Scott James Remnant <scott@netsplit.com>.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -57,52 +57,18 @@
 #include "com.ubuntu.Upstart.Job.h"
 
 
-/**
- * JOB_DEFAULT_KILL_TIMEOUT:
- *
- * The default length of time to wait after sending a process the TERM
- * signal before sending the KILL signal if it hasn't terminated.
- **/
-#define JOB_DEFAULT_KILL_TIMEOUT 5
-
-/**
- * JOB_DEFAULT_RESPAWN_LIMIT:
- *
- * The default number of times in JOB_DEFAULT_RESPAWN_INTERVAL seconds that
- * we permit a process to respawn before stoping it
- **/
-#define JOB_DEFAULT_RESPAWN_LIMIT 10
-
-/**
- * JOB_DEFAULT_RESPAWN_INTERVAL:
- *
- * The default number of seconds before resetting the respawn timer.
- **/
-#define JOB_DEFAULT_RESPAWN_INTERVAL 5
-
-/**
- * JOB_DEFAULT_UMASK:
- *
- * The default file creation mark for processes.
- **/
-#define JOB_DEFAULT_UMASK 022
-
-/**
- * JOB_DEFAULT_ENVIRONMENT:
- *
- * Environment variables to always copy from our own environment, these
- * can be overriden in the job definition or by events since they have the
- * lowest priority.
- **/
-#define JOB_DEFAULT_ENVIRONMENT \
-	"PATH",			\
-	"TERM"
-
-
 /* Prototypes for static functions */
 static void job_class_add    (JobClass *class);
 static int  job_class_remove (JobClass *class, const Session *session);
 
+/**
+ * default_console:
+ *
+ * If a job does not specify a value for the 'console' stanza, use this value.
+ *
+ * Only used if value is >= 0;
+ **/
+int default_console = -1;
 
 /**
  * job_classes:
@@ -246,11 +212,11 @@ job_class_new (const void *parent,
 	class->normalexit = NULL;
 	class->normalexit_len = 0;
 
-	class->console = CONSOLE_NONE;
+	class->console = default_console >= 0 ? default_console : CONSOLE_LOG;
 
 	class->umask = JOB_DEFAULT_UMASK;
-	class->nice = 0;
-	class->oom_score_adj = 0;
+	class->nice = JOB_DEFAULT_NICE;
+	class->oom_score_adj = JOB_DEFAULT_OOM_SCORE_ADJ;
 
 	for (i = 0; i < RLIMIT_NLIMITS; i++)
 		class->limits[i] = NULL;
@@ -258,9 +224,14 @@ job_class_new (const void *parent,
 	class->chroot = NULL;
 	class->chdir = NULL;
 
+	class->setuid = NULL;
+	class->setgid = NULL;
+
 	class->deleted = FALSE;
 	class->debug   = FALSE;
 
+	class->usage = NULL;
+
 	return class;
 
 error:
@@ -598,12 +569,23 @@ job_class_get_instance (JobClass        *class,
 	name = environ_expand (NULL, class->instance, instance_env);
 	if (! name) {
 		NihError *error;
+		nih_local char *error_message = NULL;
 
 		error = nih_error_get ();
 		if (error->number != ENOMEM) {
 			error = nih_error_steal ();
+			error_message = nih_strdup (NULL, error->message);
+			if (! error_message)
+				nih_return_system_error (-1);
+			if (class->usage) {
+				if (! nih_strcat_sprintf (&error_message, NULL,
+							"\n%s: %s", _("Usage"), class->usage)) {
+					nih_return_system_error (-1);
+				}
+			}
+
 			nih_dbus_error_raise (DBUS_ERROR_INVALID_ARGS,
-					      error->message);
+					      error_message);
 			nih_free (error);
 		}
 
@@ -797,12 +779,22 @@ job_class_start (JobClass        *class,
 	name = environ_expand (NULL, class->instance, start_env);
 	if (! name) {
 		NihError *error;
+		nih_local char *error_message = NULL;
 
 		error = nih_error_get ();
 		if (error->number != ENOMEM) {
 			error = nih_error_steal ();
+			error_message = nih_strdup (NULL, error->message);
+			if (! error_message)
+				nih_return_system_error (-1);
+			if (class->usage) {
+				if (! nih_strcat_sprintf (&error_message, NULL,
+							"\n%s: %s", _("Usage"), class->usage)) {
+					nih_return_system_error (-1);
+				}
+			}
 			nih_dbus_error_raise (DBUS_ERROR_INVALID_ARGS,
-					      error->message);
+					      error_message);
 			nih_free (error);
 		}
 
@@ -1449,3 +1441,62 @@ job_class_get_emits (JobClass *      class,
 
 	return 0;
 }
+
+/**
+ * job_class_console_type:
+ * @console: string representing console type.
+ *
+ * Returns: ConsoleType equivalent of @string, or -1 on invalid @console.
+ **/
+ConsoleType
+job_class_console_type (const char *console)
+{
+	if (! strcmp (console, "none")) {
+		return CONSOLE_NONE;
+	} else if (! strcmp (console, "output")) {
+		return CONSOLE_OUTPUT;
+	} else if (! strcmp (console, "owner")) {
+		return CONSOLE_OWNER;
+	} else if (! strcmp (console, "log")) {
+		return CONSOLE_LOG;
+	}
+
+	return (ConsoleType)-1;
+}
+
+/**
+ * job_class_get_usage:
+ * @class: class to obtain usage from,
+ * @message: D-Bus connection and message received,
+ * @usage: pointer for reply string.
+ *
+ * Implements the get method for the usage property of the
+ * com.ubuntu.Upstart.Job interface.
+ *
+ * Called to obtain the usage of the given @class
+ * which will be stored as an string in @usage.
+ *
+ * Returns: zero on success, negative value on raised error.
+ **/
+int
+job_class_get_usage (JobClass *      class,
+		     NihDBusMessage *message,
+		     char **        usage)
+{
+	nih_assert (class != NULL);
+	nih_assert (message != NULL);
+	nih_assert (usage != NULL);
+
+	if (class->usage) {
+		*usage = nih_strdup (message, class->usage);
+		}
+	else {
+		*usage = nih_strdup (message, "");
+	}
+
+	if (! *usage) {
+		nih_return_no_memory_error (-1);
+	}
+
+	return 0;
+}
