@@ -209,6 +209,14 @@ static int stanza_chdir       (JobClass *class, NihConfigStanza *stanza,
 			       const char *file, size_t len,
 			       size_t *pos, size_t *lineno)
 	__attribute__ ((warn_unused_result));
+static int stanza_setuid      (JobClass *class, NihConfigStanza *stanza,
+			       const char *file, size_t len,
+			       size_t *pos, size_t *lineno)
+	__attribute__ ((warn_unused_result));
+static int stanza_setgid      (JobClass *class, NihConfigStanza *stanza,
+			       const char *file, size_t len,
+			       size_t *pos, size_t *lineno)
+	__attribute__ ((warn_unused_result));
 static int stanza_debug       (JobClass *class, NihConfigStanza *stanza,
 			       const char *file, size_t len,
 			       size_t *pos, size_t *lineno)
@@ -217,6 +225,11 @@ static int stanza_manual      (JobClass *class, NihConfigStanza *stanza,
 			       const char *file, size_t len,
 			       size_t *pos, size_t *lineno)
 	__attribute__ ((warn_unused_result));
+static int stanza_usage       (JobClass *class, NihConfigStanza *stanza,
+			       const char *file, size_t len,
+			       size_t *pos, size_t *lineno)
+	__attribute__ ((warn_unused_result));
+
 
 
 /**
@@ -253,8 +266,11 @@ static NihConfigStanza stanzas[] = {
 	{ "limit",       (NihConfigHandler)stanza_limit       },
 	{ "chroot",      (NihConfigHandler)stanza_chroot      },
 	{ "chdir",       (NihConfigHandler)stanza_chdir       },
+	{ "setuid",      (NihConfigHandler)stanza_setuid      },
+	{ "setgid",      (NihConfigHandler)stanza_setgid      },
 	{ "debug",       (NihConfigHandler)stanza_debug       },
 	{ "manual",      (NihConfigHandler)stanza_manual      },
+	{ "usage",       (NihConfigHandler)stanza_usage       },
 
 	NIH_CONFIG_LAST
 };
@@ -2116,15 +2132,11 @@ stanza_console (JobClass        *class,
 	if (! arg)
 		goto finish;
 
-	if (! strcmp (arg, "none")) {
-		class->console = CONSOLE_NONE;
-	} else if (! strcmp (arg, "output")) {
-		class->console = CONSOLE_OUTPUT;
-	} else if (! strcmp (arg, "owner")) {
-		class->console = CONSOLE_OWNER;
-	} else {
+	class->console = job_class_console_type (arg);
+
+	if (class->console == (ConsoleType)-1) {
 		nih_return_error (-1, NIH_CONFIG_UNKNOWN_STANZA,
-				  _(NIH_CONFIG_UNKNOWN_STANZA_STR));
+				_(NIH_CONFIG_UNKNOWN_STANZA_STR));
 	}
 
 	ret = nih_config_skip_comment (file, len, &a_pos, &a_lineno);
@@ -2538,3 +2550,114 @@ stanza_chdir (JobClass        *class,
 
 	return nih_config_skip_comment (file, len, pos, lineno);
 }
+
+/**
+ * stanza_setuid:
+ * @class: job class being parsed,
+ * @stanza: stanza found,
+ * @file: file or string to parse,
+ * @len: length of @file,
+ * @pos: offset within @file,
+ * @lineno: line number.
+ *
+ * Parse a setuid stanza from @file, extracting a single argument
+ * containing a user name.
+ *
+ * Returns: zero on success, negative value on error.
+ **/
+static int
+stanza_setuid (JobClass        *class,
+	       NihConfigStanza *stanza,
+	       const char      *file,
+	       size_t           len,
+	       size_t          *pos,
+	       size_t          *lineno)
+{
+	nih_assert (class != NULL);
+	nih_assert (stanza != NULL);
+	nih_assert (file != NULL);
+	nih_assert (pos != NULL);
+
+	if (class->setuid)
+		nih_unref (class->setuid, class);
+
+	class->setuid = nih_config_next_arg (class, file, len, pos, lineno);
+	if (! class->setuid)
+		return -1;
+
+	return nih_config_skip_comment (file, len, pos, lineno);
+}
+
+/**
+ * stanza_setgid:
+ * @class: job class being parsed,
+ * @stanza: stanza found,
+ * @file: file or string to parse,
+ * @len: length of @file,
+ * @pos: offset within @file,
+ * @lineno: line number.
+ *
+ * Parse a setgid stanza from @file, extracting a single argument
+ * containing a group name.
+ *
+ * Returns: zero on success, negative value on error.
+ **/
+static int
+stanza_setgid (JobClass        *class,
+	       NihConfigStanza *stanza,
+	       const char      *file,
+	       size_t           len,
+	       size_t          *pos,
+	       size_t          *lineno)
+{
+	nih_assert (class != NULL);
+	nih_assert (stanza != NULL);
+	nih_assert (file != NULL);
+	nih_assert (pos != NULL);
+
+	if (class->setgid)
+		nih_unref (class->setgid, class);
+
+	class->setgid = nih_config_next_arg (class, file, len, pos, lineno);
+	if (! class->setgid)
+		return -1;
+
+	return nih_config_skip_comment (file, len, pos, lineno);
+}
+
+/**
+ * stanza_usage:
+ * @class: job class being parsed,
+ * @stanza: stanza found,
+ * @file: file or string to parse,
+ * @len: length of @file,
+ * @pos: offset within @file,
+ * @lineno: line number.
+ *
+ * Parse a usage stanza from @file, extracting a single argument
+ * containing a usage message.
+ *
+ * Returns: zero on success, negative value on error.
+ **/
+static int
+stanza_usage (JobClass        *class,
+	      NihConfigStanza *stanza,
+	      const char      *file,
+	      size_t           len,
+	      size_t          *pos,
+	      size_t          *lineno)
+{
+	nih_assert (class != NULL);
+	nih_assert (stanza != NULL);
+	nih_assert (file != NULL);
+	nih_assert (pos != NULL);
+
+	if (class->usage)
+		nih_unref (class->usage, class);
+
+	class->usage = nih_config_next_arg (class, file, len, pos, lineno);
+	if (! class->usage)
+		return -1;
+
+	return nih_config_skip_comment (file, len, pos, lineno);
+}
