@@ -1325,6 +1325,9 @@ int
 main (int   argc,
       char *argv[])
 {
+	/* run tests in legacy (pre-session support) mode */
+	setenv ("UPSTART_NO_SESSIONS", "1", 1);
+
 	test_operator_new ();
 	test_operator_copy ();
 	test_operator_destroy ();
